@@ -1,8 +1,8 @@
+import warnings
 from typing import Optional, Any, List, Sequence, Dict
 
 from pydantic import BaseModel
 
-from kikyo_api.errors import KikyoClientError
 from kikyo_api.nsclient.base import NamespacedClient
 from kikyo_api.serializer import JsonSerializer
 from kikyo_api.utils import read_json_data
@@ -17,17 +17,18 @@ class SearchClient(NamespacedClient):
 
     api_prefix = '/api/search'
 
-    def search(self, index: str) -> 'IndexedQuery':
+    def search(self, index: str) -> 'DataIndex':
         """
-        对指定index构建查询
+        指定索引
 
         :param index: index名称
         """
-        return IndexedQuery(index, self)
+        warnings.warn('"search" is deprecated, use "index" instead', DeprecationWarning)
+        return DataIndex(index, self)
 
     def index(self, index: str) -> 'DataIndex':
         """
-        对指定index返回索引
+        指定索引
 
         :param index: index名称
         """
@@ -48,103 +49,6 @@ class IndexedQuery:
         self._page = None
         self._size = None
 
-    def query(self, name: str = None) -> 'QueryBuilder':
-        """
-        基于筛选表达式检索数据
-
-        :param name: 筛选的字段名称
-        """
-
-        return QueryBuilder(name, self, self._queries)
-
-    def filter(self, name: str = None) -> 'QueryBuilder':
-        """
-        基于筛选表达式检索数据
-
-        :param name: 筛选的字段名称
-        """
-
-        return QueryBuilder(name, self, self._filters)
-
-    def paginate(self, page: int = 0, size: int = 10) -> 'IndexedQuery':
-        """
-        分页查询
-
-        :param page: 分页的页码，从0开始
-        :param size: 分页的大小
-        """
-
-        self._page = page
-        self._size = size
-        return self
-
-    def hits(self) -> List[dict]:
-        """
-        返回命中查询的所有数据，默认进行了分页。
-        """
-
-        body = {
-            'query': self._build_query(),
-        }
-        if self._size is not None:
-            body['size'] = self._size
-            if self._page is not None:
-                body['from'] = self._page * self._size
-
-        return self.search_by_dsl(body).hits()
-
-    def count(self) -> int:
-        """
-        返回命中查询的数据量
-        """
-
-        body = {
-            'query': self._build_query(),
-        }
-        return self.count_by_dsl(body).count()
-
-    def _build_query(self) -> dict:
-        query = BoolQuery()
-        for q in self._queries:
-            q.build(query.must, query.must_not)
-        for f in self._filters:
-            f.build(query.filter, query.must_not)
-        return {
-            'bool': query.dict()
-        }
-
-    def search_by_dsl(self, query_body: dict) -> 'Response':
-        data = read_json_data(
-            self._client.session.post(
-                f'{self._client.api_prefix}/{self._index}/search-by-dsl',
-                data=_json.dumps(
-                    {
-                        'body': query_body,
-                    }
-                ).encode(),
-                headers={
-                    'content-type': _json.mimetype,
-                },
-            )
-        )
-        return Response(data)
-
-    def count_by_dsl(self, query_body: dict) -> 'Response':
-        data = read_json_data(
-            self._client.session.post(
-                f'{self._client.api_prefix}/{self._index}/count-by-dsl',
-                data=_json.dumps(
-                    {
-                        'body': query_body,
-                    },
-                ).encode(),
-                headers={
-                    'content-type': _json.mimetype,
-                },
-            )
-        )
-        return Response(data)
-
 
 class Response:
     def __init__(self, data: dict):
@@ -162,6 +66,9 @@ class Response:
     def hits(self) -> List[dict]:
         return [h['_source'] for h in self['hits']['hits']]
 
+    def raw_hits(self) -> List[dict]:
+        return self['hits']
+
     def count(self) -> int:
         return self['count']
 
@@ -185,16 +92,12 @@ class DataIndex:
         :param id: 数据ID
         """
 
-        try:
-            read_json_data(
-                self._client.session.head(
-                    f'{self._client.api_prefix}/{self._index}/{id}',
-                )
-            )
-        except KikyoClientError as e:
-            if e.status_code == 404:
-                return False
-            raise
+        resp = self._client.session.head(
+            f'{self._client.api_prefix}/{self._index}/{id}',
+        )
+        if resp.status_code == 404:
+            return False
+        resp.raise_for_status()
         return True
 
     def get(self, id: str) -> dict:
@@ -246,6 +149,121 @@ class DataIndex:
             )
         )
 
+    def search_by_dsl(self, query_body: dict) -> 'Response':
+        data = read_json_data(
+            self._client.session.post(
+                f'{self._client.api_prefix}/{self._index}/search-by-dsl',
+                data=_json.dumps(
+                    {
+                        'body': query_body,
+                    }
+                ).encode(),
+                headers={
+                    'content-type': _json.mimetype,
+                },
+            )
+        )
+        return Response(data)
+
+    def count_by_dsl(self, query_body: dict) -> 'Response':
+        data = read_json_data(
+            self._client.session.post(
+                f'{self._client.api_prefix}/{self._index}/count-by-dsl',
+                data=_json.dumps(
+                    {
+                        'body': query_body,
+                    },
+                ).encode(),
+                headers={
+                    'content-type': _json.mimetype,
+                },
+            )
+        )
+        return Response(data)
+
+    def query(self, name: str = None) -> 'QueryBuilder':
+        """
+        基于筛选表达式检索数据，影响评分
+
+        :param name: 筛选的字段名称
+        """
+
+        q = IndexedQuery(self._index, self._client)
+        return QueryBuilder(name, q, q._queries)
+
+    def filter(self, name: str = None) -> 'QueryBuilder':
+        """
+        基于筛选表达式检索数据，不影响评分
+
+        :param name: 筛选的字段名称
+        """
+
+        q = IndexedQuery(self._index, self._client)
+        return QueryBuilder(name, q, q._filters)
+
+
+class IndexedQuery(DataIndex):
+    def __init__(self, index: str, client: SearchClient):
+        super().__init__(index, client)
+
+        self._queries: List[NamedClause] = []
+        self._filters: List[NamedClause] = []
+        self._page = None
+        self._size = None
+
+    def query(self, name: str = None) -> 'QueryBuilder':
+        return QueryBuilder(name, self, self._queries)
+
+    def filter(self, name: str = None) -> 'QueryBuilder':
+        return QueryBuilder(name, self, self._queries)
+
+    def paginate(self, page: int = 0, size: int = 10) -> 'IndexedQuery':
+        """
+        分页查询
+
+        :param page: 分页的页码，从0开始
+        :param size: 分页的大小
+        """
+
+        self._page = page
+        self._size = size
+        return self
+
+    def hits(self) -> List[dict]:
+        """
+        返回命中查询的所有数据，默认进行了分页。
+        """
+
+        body = {
+            'query': self._build_query(),
+        }
+        if self._size is not None:
+            body['size'] = self._size
+            if self._page is not None:
+                body['from'] = self._page * self._size
+
+        return self.search_by_dsl(body).hits()
+
+    def count(self) -> int:
+        """
+        返回命中查询的数据量
+        """
+
+        body = {
+            'query': self._build_query(),
+        }
+        return self.count_by_dsl(body).count()
+
+    def _build_query(self) -> dict:
+        query = BoolQuery()
+        for q in self._queries:
+            q.build(query.must, query.must_not)
+        for f in self._filters:
+            f.build(query.filter, query.must_not)
+        return {
+            'bool': query.dict()
+        }
+
 
 class BoolQuery(BaseModel):
     must: List[Dict] = []
